@@ -1,31 +1,32 @@
 # -*- coding: utf-8 -*-
 """Module to generate a full stack with VPC & Cluster."""
 
-import json
-
 import boto3
 from troposphere import Ref, GetAtt, If, Join
 from troposphere.cloudformation import Stack
 from troposphere.ecs import Cluster
 
 from ecs_composex import XFILE_DEST
-from ecs_composex.common import LOG, add_parameters, validate_kwargs
+from ecs_composex.common import (
+    LOG, add_parameters, validate_kwargs, build_parameters_file,
+    build_default_stack_parameters
+)
 from ecs_composex.common import (
     build_template, KEYISSET,
     load_composex_file
 )
 from ecs_composex.common.cfn_params import (
-    ROOT_STACK_NAME_T
+    ROOT_STACK_NAME_T,
+    USE_FLEET, USE_FLEET_T,
+    USE_ONDEMAND, USE_ONDEMAND_T
 )
 from ecs_composex.common.tagging import generate_tags_parameters, add_object_tags
 from ecs_composex.common.templates import upload_template
-from ecs_composex.compute import create_cluster_template
-from ecs_composex.compute.cluster_params import (
+from ecs_composex.compute import create_compute_stack
+from ecs_composex.compute.compute_params import (
     TARGET_CAPACITY_T,
     TARGET_CAPACITY,
-    MIN_CAPACITY_T,
-    USE_FLEET, USE_FLEET_T,
-    USE_ONDEMAND, USE_ONDEMAND_T
+    MIN_CAPACITY_T
 )
 from ecs_composex.ecs import ecs_params, create_services_templates
 from ecs_composex.ecs.ecs_conditions import (
@@ -44,28 +45,6 @@ from ecs_composex.vpc import vpc_params
 ROOT_CLUSTER_NAME = 'EcsCluster'
 
 
-def build_parameters_file(params: list, parameter_name: str, parameter_value):
-    """
-    Function to build arguments file to pass onto CFN.
-    Adds the parameter key/value so it can be written to file afterwards
-
-    :param params: list of parameters
-    :type params: list
-    :param parameter_name: key of the parameter
-    :type parameter_name: str
-    :param parameter_value: value of the parameter
-    :type parameter_value: str or int or list
-    """
-    if params is None:
-        params = []
-    if isinstance(parameter_value, (int, float)):
-        parameter_value = str(parameter_value)
-    params.append({
-        "ParameterKey": parameter_name,
-        "ParameterValue": parameter_value
-    })
-
-
 def generate_vpc_parameters(template, params, **kwargs):
     """Function to add the VPC arguments to the root stack
     :param params: list of parameters
@@ -109,6 +88,8 @@ def add_vpc_to_root(root_template, session, tags_params=None, **kwargs):
     :return: vpc_stack
     :rtype: troposphere.cloudformation.Stack
     """
+    if tags_params is None:
+        tags_params = ()
     vpc_template = create_vpc_template(
         session=session,
         **kwargs
@@ -137,24 +118,38 @@ def add_vpc_to_root(root_template, session, tags_params=None, **kwargs):
     return vpc_stack
 
 
-def add_cluster_to_root(root_template, params=None, vpc_stack=None, session=None, **kwargs):
+def add_compute(root_template, dependencies, params, vpc_stack=None, tags=None, session=None, **kwargs):
     """
-    Function to add Cluster stack to root one.
+    Function to add Cluster stack to root one. If any of the options related to compute resources are set in the CLI
+    then this function will generate and add the compute template to the root stack template
 
+    :param dependencies: list of dependencies that need created before creating the Compute stack
+    :type dependencies: list
     :param root_template: the root template
     :type root_template: troposphere.Template
-    :param params:
-    :param vpc_stack:
-    :param session:
-    :param kwargs:
+    :param params: list of parameters
+    :param vpc_stack: the VPC stack if any to pull the attributes from
+    :param session: override boto3 session
+    :param tags: tags to add to the stack
+    :type tags: tuple
 
-    :return:
+    :return: compute_stack, the Compute stack
+    :rtype: troposphere.cloudformation.Stack
     """
+    create_compute = False
+    args = [USE_FLEET_T, USE_ONDEMAND_T, "AddComputeResources"]
+    for arg in args:
+        if KEYISSET(arg, kwargs):
+            create_compute = True
+    if not create_compute:
+        return
     if params is None:
         params = []
+    if tags is None:
+        tags = ()
     depends_on = []
     root_template.add_parameter(TARGET_CAPACITY)
-    cluster_template = create_cluster_template(session, **kwargs)
+    compute_template = create_compute_stack(session, **kwargs)
     parameters = {
         ROOT_STACK_NAME_T: Ref('AWS::StackName'),
         TARGET_CAPACITY_T: Ref(TARGET_CAPACITY),
@@ -162,6 +157,8 @@ def add_cluster_to_root(root_template, params=None, vpc_stack=None, session=None
         USE_FLEET_T: Ref(USE_FLEET),
         USE_ONDEMAND_T: Ref(USE_ONDEMAND)
     }
+    for tag in tags[0]:
+        parameters.update({tag.title: Ref(tag.title)})
     if vpc_stack is not None:
         depends_on.append(vpc_stack)
         parameters.update({
@@ -169,29 +166,30 @@ def add_cluster_to_root(root_template, params=None, vpc_stack=None, session=None
             vpc_params.APP_SUBNETS_T: GetAtt(vpc_stack, f"Outputs.{vpc_params.APP_SUBNETS_T}"),
         })
     else:
-        # Setup parameters for cluster stack without VPC pre-created
-        if not kwargs['AppSubnets']:
-            raise ValueError("No application subnets were provided to create the cluster")
-        elif params is None:
+        # Setup parameters for compute stack without VPC pre-created
+        if not kwargs[vpc_params.APP_SUBNETS_T]:
+            raise ValueError("No application subnets were provided to create the compute")
+        if params is None:
             raise TypeError("params is ", params, "expected", list)
         root_template.add_parameter(vpc_params.APP_SUBNETS)
         parameters.update({
             vpc_params.APP_SUBNETS_T: Ref(vpc_params.APP_SUBNETS)
         })
-        build_parameters_file(params, vpc_params.APP_SUBNETS_T, kwargs['AppSubnets'])
+        build_parameters_file(params, vpc_params.APP_SUBNETS_T, kwargs[vpc_params.APP_SUBNETS_T])
 
-    cluster_stack = root_template.add_resource(Stack(
+    compute_stack = root_template.add_resource(Stack(
         'Compute',
         TemplateURL=upload_template(
-            template_body=cluster_template.to_json(),
+            template_body=compute_template[0].to_json(),
             bucket_name=kwargs['BucketName'],
-            file_name='cluster.json',
-            session=session
+            file_name='compute.json',
+            session=session,
         ),
-        Parameters=parameters
-    )
-    )
-    return cluster_stack
+        Parameters=parameters,
+        DependsOn=dependencies
+    ))
+    dependencies.append('Compute')
+    return compute_stack
 
 
 def add_x_resources(template, session, tags=None, **kwargs):
@@ -261,7 +259,6 @@ def add_services(template, depends, session, vpc_stack=None, **kwargs):
             vpc_params.VPC_ID_T: GetAtt(vpc_stack, f'Outputs.{vpc_params.VPC_ID_T}'),
             vpc_params.PUBLIC_SUBNETS_T: GetAtt(vpc_stack, f'Outputs.{vpc_params.PUBLIC_SUBNETS_T}'),
             vpc_params.APP_SUBNETS_T: GetAtt(vpc_stack, f'Outputs.{vpc_params.APP_SUBNETS_T}'),
-
         })
     else:
         parameters.update({
@@ -303,14 +300,19 @@ def add_ecs_cluster(template, depends_on=None):
     )
 
 
-def init_root_template(session, stack_params, tags=None, **kwargs):
+def init_root_template(stack_params, tags=None):
     """
     Function to initialize the root template
-    :param session:
-    :param tags_params:
-    :param kwargs:
-    :return:
+
+    :param stack_params: stack parameters
+    :type stack_params: list
+    :param tags: tags and parameters to add to the template
+    :type tags: tuple
+
+    :return: template
+    :rtype: troposphere.Template
     """
+
     template = build_template(
         'Root template generated via ECS ComposeX',
         [USE_FLEET, USE_ONDEMAND, CLUSTER_NAME]
@@ -334,16 +336,16 @@ def generate_full_template(session=None, **kwargs):
     if session is None:
         session = boto3.session.Session(region_name=kwargs['AwsRegion'])
     stack_params = []
+    build_default_stack_parameters(stack_params, **kwargs)
     tags_params = generate_tags_parameters(load_composex_file(kwargs[XFILE_DEST]))
-    template = init_root_template(session, stack_params, tags_params, **kwargs)
+    template = init_root_template(stack_params, tags_params)
     LOG.debug(kwargs)
     validate_kwargs(['BucketName'], kwargs)
-
     vpc_stack = None
-    depends = add_x_resources(template, session=session, **kwargs)
+    depends_on = add_x_resources(template, session=session, **kwargs)
     if KEYISSET('CreateVpc', kwargs):
         vpc_stack = add_vpc_to_root(template, session, tags_params[0], **kwargs)
-        depends.append(vpc_stack)
+        depends_on.append(vpc_stack)
         add_object_tags(vpc_stack, tags_params[1])
     else:
         generate_vpc_parameters(template, stack_params, **kwargs)
@@ -351,12 +353,11 @@ def generate_full_template(session=None, **kwargs):
     if KEYISSET(CLUSTER_NAME_T, kwargs):
         build_parameters_file(stack_params, CLUSTER_NAME_T, kwargs[CLUSTER_NAME_T])
     if KEYISSET('CreateCluster', kwargs):
-        add_ecs_cluster(template, depends)
-        depends.append(ROOT_CLUSTER_NAME)
-    add_services(template, depends, session=session, vpc_stack=vpc_stack, **kwargs)
+        add_ecs_cluster(template, depends_on)
+        depends_on.append(ROOT_CLUSTER_NAME)
+    add_compute(template, depends_on, stack_params, vpc_stack, tags=tags_params, session=session, **kwargs)
+    add_services(template, depends_on, session=session, vpc_stack=vpc_stack, **kwargs)
+
     for resource in template.resources:
         add_object_tags(template.resources[resource], tags_params[1])
-    with open(kwargs['output_file'], 'w') as root_fd:
-        root_fd.write(template.to_json())
-    with open(f"{kwargs['output_file'].split('.')[0]}.params.json", 'w') as params_fd:
-        params_fd.write(json.dumps(stack_params, indent=4))
+    return template, stack_params
